/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endour or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package org.seasar.kijimuna.core.dicon.tools;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IStorage;
import org.seasar.kijimuna.core.ConstCore;
import org.seasar.kijimuna.core.dicon.model.ArgElement;
import org.seasar.kijimuna.core.dicon.model.AspectElement;
import org.seasar.kijimuna.core.dicon.model.ComponentElement;
import org.seasar.kijimuna.core.dicon.model.ContainerElement;
import org.seasar.kijimuna.core.dicon.model.DescriptionElement;
import org.seasar.kijimuna.core.dicon.model.DestroyMethodElement;
import org.seasar.kijimuna.core.dicon.model.IncludeElement;
import org.seasar.kijimuna.core.dicon.model.InitMethodElement;
import org.seasar.kijimuna.core.dicon.model.PropertyElement;
import org.seasar.kijimuna.core.parser.Element;
import org.seasar.kijimuna.core.parser.ElementFactory;

/**
 * @author Masataka Kurihara (Gluegent, Inc.)
 */
public class DiconElementFactory extends ElementFactory
		implements ConstCore {

	public Element createElement(
	        IProject project, IStorage storage, String elementName) {
		Element element = null;
		if (DICON_TAG_CONTAINER.equals(elementName)) {
			element = new ContainerElement(project, storage);
		} else if (DICON_TAG_COMPONENT.equals(elementName)) {
			element = new ComponentElement(project, storage);
		} else if (DICON_TAG_ARG.equals(elementName)) {
			element = new ArgElement(project, storage);
		} else if (DICON_TAG_ASPECT.equals(elementName)) {
			element = new AspectElement(project, storage);
		} else if (DICON_TAG_PROPERTY.equals(elementName)) {
			element = new PropertyElement(project, storage);
		} else if (DICON_TAG_INITMETHOD.equals(elementName)) {
			element = new InitMethodElement(project, storage);
		} else if (DICON_TAG_DESTROYMETHOD.equals(elementName)) {
			element = new DestroyMethodElement(project, storage);
		} else if (DICON_TAG_INCLUDE.equals(elementName)) {
			element = new IncludeElement(project, storage);
		} else if(DICON_TAG_DESCRIPTION.equals(elementName)){
			element = new DescriptionElement(project, storage);
		} else {
			element = super.createElement(project, storage, elementName);
		}
		return element;
	}

}